/*  PNMNORMALISE - normalise each part of a picture individually
 *
 * This file is part of PhotoPNMTools,
 * Release $Name: rel-1-3-0 $,
 * File rev. $Id: pnmnormalise.c,v 1.2 2003/03/08 17:17:21 schooten Exp $.
 *
 * Copyright (c) 2002, 2003 by B.W. van Schooten. All rights reserved.
 * This software is distributed under the Perl Artistic License.
 * No warranty. See file 'artistic.license' for more details.
 *
 * boris@13thmonkey.org
 * www.13thmonkey.org/~boris/photopnmtools/
 */

#include <pnm.h>
#include <stdio.h>
#include <math.h>

#define MAX(a,b) ( (a) > (b) ? (a) : (b) )

#define USAGE \
"pnmnormalise: normalise each part of a picture individually.\n" \
"A rough filter that can be used to normalise the brightness of a\n" \
"photographed page of text, so that it can be thresholded to obtain a\n" \
"B/W bitmap.\n"\
"It samples the pixels around each pixel, and renormalises the r,g,b of\n"\
"the pixel accordingly.\n"\
"options: -a [area size, >1, default=128]\n" \
"options: -s [step size, >=1, default=8]\n" \
"\n"



	int
main(int argc, char **argv) {
	xel **in,**out;
	int area=15, step=3;
	float r,g,b;
	char optch;

	pixval maxpix;
	int format;
	int xsize,ysize;
	int x,y, dx,dy, x2,y2;
	int nrpix;
	float totr,totg,totb;
	float rmin,gmin,bmin;
	float rmax,gmax,bmax;
	pnm_init(&argc,argv);
	while ( (optch = getopt(argc, argv, "a:s:")) != -1) {
		switch (optch) {
		case 'a': area=atol(optarg);
		break;
		case 's': step=atol(optarg);
		break;
		}
	}
	argc -= optind;
	argv += optind;
	if (argc != 0) {
		fprintf(stderr,USAGE);
		exit(EXIT_FAILURE);
	}

	in = pnm_readpnm(stdin,&xsize,&ysize,&maxpix,&format);
	out = pnm_allocarray(xsize,ysize);

	for (y=0; y<ysize; y++) {
		fprintf(stderr,".");
		for (x=0; x<xsize; x++) {
			if (x%step == 0) {
				/* recalculate min,mid,max */
				nrpix=0;
				totr=0; rmax=0; rmin=maxpix;
				totg=0; gmax=0; gmin=maxpix;
				totb=0; bmax=0; bmin=maxpix;
				for (dx=-area; dx<=area; dx+=step) {
					x2 = x+dx;
					if (x2 < 0 || x2 >= xsize) continue;
					for (dy=-area; dy<=area; dy+=step) {
						y2 = y+dy;
						if (y2 < 0 || y2 >= ysize) continue;
						nrpix++;
						totr += in[y2][x2].r;
						if (in[y2][x2].r < rmin) rmin=in[y2][x2].r;
						if (in[y2][x2].r > rmax) rmax=in[y2][x2].r;
						totg += in[y2][x2].g;
						if (in[y2][x2].g < gmin) gmin=in[y2][x2].g;
						if (in[y2][x2].g > gmax) gmax=in[y2][x2].g;
						totb += in[y2][x2].b;
						if (in[y2][x2].b < bmin) bmin=in[y2][x2].b;
						if (in[y2][x2].b > bmax) bmax=in[y2][x2].b;
					}
				}
			}
			r = in[y][x].r;
			if (r < rmin) r=rmin;
			if (r > rmax) r=rmax;
			g = in[y][x].g;
			if (g < gmin) g=gmin;
			if (g > gmax) g=gmax;
			b = in[y][x].b;
			if (b < bmin) b=bmin;
			if (b > bmax) b=bmax;
			if (rmin<rmax) {
				r -= rmin;
				r *= (maxpix-1.0) / (rmax-rmin);
			}
			if (gmin<gmax) {
				g -= gmin;
				g *= (maxpix-1.0) / (gmax-gmin);
			}
			if (bmin<bmax) {
				b -= bmin;
				b *= (maxpix-1.0) / (bmax-bmin);
			}
			out[y][x].r = r;
			out[y][x].g = g;
			out[y][x].b = b;
		}
	}

	pnm_writepnm(stdout,out,xsize,ysize,maxpix,format,0/*forceplain*/);
	pnm_freearray(in,ysize);
	pnm_freearray(out,ysize);
	exit(EXIT_SUCCESS);
	return 0;
}

