/*  PPMLEVELS - levels tool for renormalising brightness
 *
 * This file is part of PhotoPNMTools,
 * Release $Name: rel-1-3-0 $,
 * File rev. $Id: pnmsharpness.c,v 1.1 2003/05/02 20:35:16 schooten Exp $.
 *
 * Copyright (c) 2002, 2003 by B.W. van Schooten. All rights reserved.
 * This software is distributed under the Perl Artistic License.
 * No warranty. See file 'artistic.license' for more details.
 *
 * boris@13thmonkey.org
 * www.13thmonkey.org/~boris/photopnmtools/
 */

#include <pnm.h>
#include <stdio.h>
#include <math.h>

#include "stdpnm.h"

#define MAX(a,b) ( (a) > (b) ? (a) : (b) )

#define USAGE \
"pnmsharpness: evaluating sharpness of a picture\n"\
"    $Id: pnmsharpness.c,v 1.1 2003/05/02 20:35:16 schooten Exp $\n"\
"    (C) 2002, 2003 Boris van Schooten   boris@13thmonkey.org\n" \
"options: -m : create sharpness map as PPM.\n"\
"\n"
/*"         -c [size, >= 1, default 1] : context size.\n"\*/


#define R 0
#define G 1
#define B 2

char *colname[] = {"red","green","blue"};



	int
main(int argc, char **argv) {
	xel **in,**out=NULL;
	struct rgb **src;

	int makemap=0;
	int contextsize=1;
	struct rgb rgb,thisrgb;
	char optch;
	
	pixval maxpix;
	int format;
	int xsize,ysize;
	int x,y, dx,dy;


	double totsharp=0.0;
	double sharpr,sharpg,sharpb;

	stdpnminit();
	pnm_init(&argc,argv);
	while ( (optch = getopt(argc, argv, "mc:")) != -1) {
		switch (optch) {
		case 'm': makemap=1;
		break;
/*		case 'c': contextsize = atoi(optarg);
		break;*/
		}
	}
	argc -= optind;
	argv += optind;
	if (argc != 0) {
		fprintf(stderr,USAGE);
		exit(EXIT_FAILURE);
	}

	in = pnm_readpnm(stdin,&xsize,&ysize,&maxpix,&format);
	src=rgballocarray(xsize,ysize, in, maxpix);
	for (y=0; y<ysize; y++) {
		for (x=0; x<xsize; x++) {
			src[y][x].r = cierectolinear(src[y][x].r,0);
			src[y][x].g = cierectolinear(src[y][x].g,1);
			src[y][x].b = cierectolinear(src[y][x].b,2);
		}
	}
	if (!makemap) {
		for (y=1; y<ysize-1; y++) {
			for (x=1; x<xsize-1; x++) {
				rgb=src[y][x];
				for (dy=-1; dy<=1; dy++) {
					for (dx=-1; dx<=1; dx++) {
						if (dx==0 && dy==0) continue;
						thisrgb=src[y+dy][x+dx];
						totsharp += fabs(thisrgb.r - rgb.r);
						totsharp += fabs(thisrgb.g - rgb.g);
						totsharp += fabs(thisrgb.b - rgb.b);
					}
				}
			}
		}
		totsharp /= xsize*ysize;
		printf("Sharpness = %f\n",totsharp);
	} else {
		out = pnm_allocarray(xsize,ysize);
		for (y=1; y<ysize-1; y++) {
			for (x=1; x<xsize-1; x++) {
				rgb=src[y][x];
				sharpr=0.0;
				sharpg=0.0;
				sharpb=0.0;
				for (dy=-1; dy<=1; dy++) {
					for (dx=-1; dx<=1; dx++) {
						if (dx==0 && dy==0) continue;
						thisrgb=src[y+dy][x+dx];
						sharpr += fabs(thisrgb.r - rgb.r);
						sharpg += fabs(thisrgb.g - rgb.g);
						sharpb += fabs(thisrgb.b - rgb.b);
					}
				}
				rgb.r=sharpr;
				rgb.g=sharpg;
				rgb.b=sharpb;
				out[y][x] = rgbtoxel(rgb,maxpix);
			}
		}
		pnm_writepnm(stdout,out,xsize,ysize,maxpix,format,0/*forceplain*/);
		pnm_freearray(out,ysize);
	}
	pnm_freearray(in,ysize);
	exit(EXIT_SUCCESS);
	return 0;
}

