/*  PPMTESTIMAGE - Generate test image for printing
 *
 * This file is part of PhotoPNMTools,
 * Release $Name: rel-1-3-0 $,
 * File rev. $Id: ppmtestimage.c,v 1.4 2003/03/13 15:50:59 schooten Exp $.
 *
 * Copyright (c) 2002, 2003 by B.W. van Schooten. All rights reserved.
 * This software is distributed under the Perl Artistic License.
 * No warranty. See file 'artistic.license' for more details.
 *
 * boris@13thmonkey.org
 * www.13thmonkey.org/~boris/photopnmtools/
 */

#include <pnm.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

#define MAX(a,b) ( (a) > (b) ? (a) : (b) )

#define USAGE \
"ppmtestimage: create test image for printing.\n" \
"Contains colour gradients for testing colour response and brightness\n"\
"clipping behaviour, and striped patterns for testing resolution and\n" \
"interpolation behaviour.  An outer margin of concentric rectancles is\n" \
"used to test cropping behaviour.\n" \
"options: -x [xsize, default=1024]\n" \
"         -y [ysize, default=768]\n" \
"         -m [outer margin in pixels, default=(xsize+ysize)/20]\n" \
"         -r [outer margin rectangle spacing in pixels, default=20]\n" \
"         -a [colour sine pattern amplitude, default=20]\n"\
"         -n [no. of striped areas, default=9]\n" \
"\n" \


	int
main(int argc, char **argv) {
	xel **out;

	int xofs,yofs;
	double xval,yval;
	char optch;

	int nr_bands=10;
	double sineamp=20;
	int rectspacing=20;

	pixval maxpix=255;
	int format=20531;
	int xsize=1024,ysize=768,margin=0;
	int x,y,z;
	int ixsize,iysize;
	pnm_init(&argc,argv);
	while ( (optch = getopt(argc, argv, "x:y:m:n:a:r:")) != -1) {
		switch (optch) {
		case 'x': xsize=atol(optarg);
		break;
		case 'y': ysize=atol(optarg);
		break;
		case 'm': margin=atol(optarg);
		break;
		case 'n': nr_bands=atol(optarg) + 1;
		break;
		case 'a': sineamp=atof(optarg);
		break;
		case 'r': rectspacing=atol(optarg);
		break;
		}
	}
	argc -= optind;
	argv += optind;
	if (argc != 0) {
		fprintf(stderr,USAGE);
		exit(EXIT_FAILURE);
	}
	if (margin == 0) {
		margin = (xsize + ysize) / 20;
	}
	//in = pnm_readpnm(stdin,&xsize,&ysize,&maxpix,&format);
	out = pnm_allocarray(xsize,ysize);

	ixsize = xsize - 2*margin;
	iysize = ysize - 2*margin;
	/* crop test margin */
	for (y=0; y<ysize; y++) {
		for (x=0; x<margin; x+=rectspacing) {
			out[y][x].r = maxpix;
			out[y][x].g = maxpix;
			out[y][x].b = maxpix;
			out[y][x+1].r = maxpix;
			out[y][x+1].g = maxpix;
			out[y][x+1].b = maxpix;
			out[y][xsize-x-1].r = maxpix;
			out[y][xsize-x-1].g = maxpix;
			out[y][xsize-x-1].b = maxpix;
			out[y][xsize-x-2].r = maxpix;
			out[y][xsize-x-2].g = maxpix;
			out[y][xsize-x-2].b = maxpix;
		}
	}

	for (x=0; x<xsize; x++) {
		for (y=0; y<margin; y+=rectspacing) {
			out[y][x].r = maxpix;
			out[y][x].g = maxpix;
			out[y][x].b = maxpix;
			out[y+1][x].r = maxpix;
			out[y+1][x].g = maxpix;
			out[y+1][x].b = maxpix;
			out[ysize-y-1][x].r = maxpix;
			out[ysize-y-1][x].g = maxpix;
			out[ysize-y-1][x].b = maxpix;
			out[ysize-y-2][x].r = maxpix;
			out[ysize-y-2][x].g = maxpix;
			out[ysize-y-2][x].b = maxpix;
		}
	}

	/* colour pattern */
	xofs=margin + 0*ixsize/6;
	yofs = margin;
	for (y=0; y<iysize/5; y++) {
		for (x=0; x<ixsize/7; x++) {
			xval = ((double)x / (ixsize/7));
			yval = ((double)y / (iysize/5));
			out[yofs+y][xofs+x].r = maxpix*xval;
			out[yofs+y][xofs+x].g = maxpix*yval;
			out[yofs+y][xofs+x].b = 0;
		}
	}
	xofs=margin + ixsize/6;
	for (y=0; y<iysize/5; y++) {
		for (x=0; x<ixsize/7; x++) {
			xval = ((double)x / (ixsize/7));
			yval = ((double)y / (iysize/5));
			out[yofs+y][xofs+x].r = maxpix*xval;
			out[yofs+y][xofs+x].g = maxpix*yval;
			out[yofs+y][xofs+x].b = maxpix;
		}
	}
	xofs=margin + 2*ixsize/6;
	for (y=0; y<iysize/5; y++) {
		for (x=0; x<ixsize/7; x++) {
			xval = ((double)x / (ixsize/7));
			yval = ((double)y / (iysize/5));
			out[yofs+y][xofs+x].r = 0;
			out[yofs+y][xofs+x].g = maxpix*xval;
			out[yofs+y][xofs+x].b = maxpix*yval;
		}
	}
	xofs=margin + 3*ixsize/6;
	for (y=0; y<iysize/5; y++) {
		for (x=0; x<ixsize/7; x++) {
			xval = ((double)x / (ixsize/7));
			yval = ((double)y / (iysize/5));
			out[yofs+y][xofs+x].r = maxpix;
			out[yofs+y][xofs+x].g = maxpix*xval;
			out[yofs+y][xofs+x].b = maxpix*yval;
		}
	}
	xofs=margin + 4*ixsize/6;
	for (y=0; y<iysize/5; y++) {
		for (x=0; x<ixsize/7; x++) {
			xval = ((double)x / (ixsize/7));
			yval = ((double)y / (iysize/5));
			out[yofs+y][xofs+x].r = maxpix*xval;
			out[yofs+y][xofs+x].g = 0;
			out[yofs+y][xofs+x].b = maxpix*yval;
		}
	}
	xofs=margin + 5*ixsize/6;
	for (y=0; y<iysize/5; y++) {
		for (x=0; x<ixsize/7; x++) {
			xval = ((double)x / (ixsize/7));
			yval = ((double)y / (iysize/5));
			out[yofs+y][xofs+x].r = maxpix*xval;
			out[yofs+y][xofs+x].g = maxpix;
			out[yofs+y][xofs+x].b = maxpix*yval;
		}
	}

	/* colour pattern + sine + random */
	yofs = margin + iysize/4;

	xofs=margin + 0*ixsize/6;
	for (y=0; y<iysize/5; y++) {
		for (x=0; x<ixsize/7; x++) {
			xval = ((double)x / (ixsize/7));
			yval = ((double)y / (iysize/5));
			out[yofs+y][xofs+x].r = (maxpix-(sineamp*2))*xval
				+ sineamp + sineamp*cos(x);
			out[yofs+y][xofs+x].g = (maxpix-(sineamp*2))*yval
				+ sineamp + sineamp*cos(x);
			out[yofs+y][xofs+x].b =
				0;//random()%(int)(sineamp*2);
		}
	}
	xofs=margin + 1*ixsize/6;
	for (y=0; y<iysize/5; y++) {
		for (x=0; x<ixsize/7; x++) {
			xval = ((double)x / (ixsize/7));
			yval = ((double)y / (iysize/5));
			out[yofs+y][xofs+x].r = (maxpix-(sineamp*2))*xval
				+ sineamp + sineamp*cos(x);
			out[yofs+y][xofs+x].g = (maxpix-(sineamp*2))*yval
				+ sineamp + sineamp*cos(x);
			out[yofs+y][xofs+x].b = (maxpix-(sineamp*2))
				;//+ random()%(int)(sineamp*2);
		}
	}
	xofs=margin + 2*ixsize/6;
	for (y=0; y<iysize/5; y++) {
		for (x=0; x<ixsize/7; x++) {
			xval = ((double)x / (ixsize/7));
			yval = ((double)y / (iysize/5));
			out[yofs+y][xofs+x].r =
				0;//random()%(int)(sineamp*2);
			out[yofs+y][xofs+x].g = (maxpix-(sineamp*2))*xval
				+ sineamp + sineamp*cos(x);
			out[yofs+y][xofs+x].b = (maxpix-(sineamp*2))*yval
				+ sineamp + sineamp*cos(x);
		}
	}
	xofs=margin + 3*ixsize/6;
	for (y=0; y<iysize/5; y++) {
		for (x=0; x<ixsize/7; x++) {
			xval = ((double)x / (ixsize/7));
			yval = ((double)y / (iysize/5));
			out[yofs+y][xofs+x].r = (maxpix-(sineamp*2))
				;//+ random()%(int)(sineamp*2);
			out[yofs+y][xofs+x].g = (maxpix-(sineamp*2))*xval
				+ sineamp + sineamp*cos(x);
			out[yofs+y][xofs+x].b = (maxpix-(sineamp*2))*yval
				+ sineamp + sineamp*cos(x);
		}
	}
	xofs=margin + 4*ixsize/6;
	for (y=0; y<iysize/5; y++) {
		for (x=0; x<ixsize/7; x++) {
			xval = ((double)x / (ixsize/7));
			yval = ((double)y / (iysize/5));
			out[yofs+y][xofs+x].r = (maxpix-(sineamp*2))*xval
				+ sineamp + sineamp*cos(x);
			out[yofs+y][xofs+x].g =
				0;//random()%(int)(sineamp*2);
			out[yofs+y][xofs+x].b = (maxpix-(sineamp*2))*yval
				+ sineamp + sineamp*cos(x);
		}
	}
	xofs=margin + 5*ixsize/6;
	for (y=0; y<iysize/5; y++) {
		for (x=0; x<ixsize/7; x++) {
			xval = ((double)x / (ixsize/7));
			yval = ((double)y / (iysize/5));
			out[yofs+y][xofs+x].r = (maxpix-(sineamp*2))*xval
				+ sineamp + sineamp*cos(x);
			out[yofs+y][xofs+x].g = (maxpix-(sineamp*2))
				;//+ random()%(int)(sineamp*2);
			out[yofs+y][xofs+x].b = (maxpix-(sineamp*2))*yval
				+ sineamp + sineamp*cos(x);
		}
	}
	xofs=margin + 4*ixsize/6;
	for (y=0; y<iysize/5; y++) {
		for (x=0; x<ixsize/7; x++) {
			xval = ((double)x / (ixsize/7));
			yval = ((double)y / (iysize/5));
			out[yofs+y][xofs+x].r = (maxpix-(sineamp*2))*xval
				+ sineamp + sineamp*cos(x);
			out[yofs+y][xofs+x].g =
				0;//random()%(int)(sineamp*2);
			out[yofs+y][xofs+x].b = (maxpix-(sineamp*2))*yval
				+ sineamp + sineamp*cos(x);
		}
	}
	xofs=margin + 5*ixsize/6;
	for (y=0; y<iysize/5; y++) {
		for (x=0; x<ixsize/7; x++) {
			xval = ((double)x / (ixsize/7));
			yval = ((double)y / (iysize/5));
			out[yofs+y][xofs+x].r = (maxpix-(sineamp*2))*xval
				+ sineamp + sineamp*cos(x);
			out[yofs+y][xofs+x].g = (maxpix-(sineamp*2))
				;//+ random()%(int)(sineamp*2);
			out[yofs+y][xofs+x].b = (maxpix-(sineamp*2))*yval
				+ sineamp + sineamp*cos(x);
		}
	}
	/* line patterns */
	for (z=1; z<nr_bands; z++) {
		xofs=margin + (z-1)*ixsize/nr_bands;
		yofs=margin + iysize/2;
		for (y=0; y<iysize/8; y++) {
			for (x=0; x<ixsize/nr_bands; x++) {
				out[yofs+y][xofs+x].r = (maxpix/2) +
					(maxpix/2)*cos((1.0/z)*((float)x*3.1415265));
				out[yofs+y][xofs+x].g = (maxpix/2) +
					(maxpix/2)*cos((1.0/z)*((float)x*3.1415265));
				out[yofs+y][xofs+x].b = (maxpix/2) +
					(maxpix/2)*cos((1.0/z)*((float)x*3.1415265));
			}
		}
	}
	for (z=1; z<nr_bands; z++) {
		xofs=margin + (z-1)*ixsize/nr_bands;
		yofs=margin + iysize/2 + iysize/8;
		for (y=0; y<iysize/8; y++) {
			for (x=0; x<ixsize/nr_bands; x++) {
				out[yofs+y][xofs+x].r = (maxpix/2) +
					(maxpix/2)*cos((1.0/z)*((float)(x+y)*3.1415265));
				out[yofs+y][xofs+x].g = (maxpix/2) +
					(maxpix/2)*cos((1.0/z)*((float)(x+y)*3.1415265));
				out[yofs+y][xofs+x].b = (maxpix/2) +
					(maxpix/2)*cos((1.0/z)*((float)(x+y)*3.1415265));
			}
		}
	}
	for (z=1; z<nr_bands; z++) {
		xofs=margin + (z-1)*ixsize/nr_bands;
		yofs=margin + iysize/2 + 2*iysize/8;
		for (y=0; y<iysize/8; y++) {
			for (x=0; x<ixsize/nr_bands; x++) {
				out[yofs+y][xofs+x].r = (maxpix/2) +
					(maxpix/2)*cos((1.0/z)*((float)(y)*3.1415265));
				out[yofs+y][xofs+x].g = (maxpix/2) +
					(maxpix/2)*cos((1.0/z)*((float)(y)*3.1415265));
				out[yofs+y][xofs+x].b = (maxpix/2) +
					(maxpix/2)*cos((1.0/z)*((float)(y)*3.1415265));
			}
		}
	}
	for (z=1; z<nr_bands; z++) {
		xofs=margin + (z-1)*ixsize/nr_bands;
		yofs=margin + iysize/2 + 3*iysize/8;
		for (y=0; y<iysize/8; y++) {
			for (x=0; x<ixsize/nr_bands; x++) {
				out[yofs+y][xofs+x].r = (maxpix/2) +
					(maxpix/2)*cos((1.0/z)*((float)(x)*3.1415265));
				out[yofs+y][xofs+x].g = (maxpix/2) +
					(maxpix/2)*cos((1.0/z)*((float)(x-y)*3.1415265));
				out[yofs+y][xofs+x].b = (maxpix/2) +
					(maxpix/2)*cos((1.0/z)*((float)(y)*3.1415265));
			}
		}
	}

	/* greyscale */
	xofs=margin + (nr_bands-1)*ixsize/nr_bands;
	yofs=margin + iysize/2;
	for (y=0; y<4*iysize/8; y++) {
		for (x=0; x<ixsize/nr_bands; x++) {
			out[yofs+y][xofs+x].r = maxpix*y/(4.0*iysize/8.0);
			out[yofs+y][xofs+x].g = maxpix*y/(4.0*iysize/8.0);
			out[yofs+y][xofs+x].b = maxpix*y/(4.0*iysize/8.0);
		}
	}

	pnm_writepnm(stdout,out,xsize,ysize,maxpix,format,0/*forceplain*/);
	//pnm_freearray(in,ysize);
	pnm_freearray(out,ysize);
	exit(EXIT_SUCCESS);
	return 0;
}

