/* Some standard functions for photopnmtools.  Note: we have c functions in a
 * header file here. 
 *
 * This file is part of PhotoPNMTools,
 * Release $Name: rel-1-3-0 $,
 * File rev. $Id: stdpnm.h,v 1.8 2003/04/09 00:13:46 schooten Exp $.
 *
 * Copyright (c) 2002, 2003 by B.W. van Schooten. All rights reserved.
 * This software is distributed under the Perl Artistic License.
 * No warranty. See file 'artistic.license' for more details.
 *
 * boris@13thmonkey.org
 * www.13thmonkey.org/~boris/photopnmtools/
 */

//#define MAX(a,b) ( (a) > (b) ? (a) : (b) )

#define MAX3(a,b,c) ( (a) > (b)   ? \
		( (a) > (c) ? (a) : (c) ) : \
		( (b) > (c) ? (b) : (c) )  )

#define MIN3(a,b,c) ( (a) < (b)   ? \
		( (a) < (c) ? (a) : (c) ) : \
		( (b) < (c) ? (b) : (c) )  )

struct rgb {
	float r,g,b;
};

	struct rgb
xeltorgb(xel rgbxel, int maxpix) {
	struct rgb rgb;
	rgb.r = (double)rgbxel.r / (double)maxpix;
	rgb.g = (double)rgbxel.g / (double)maxpix;
	rgb.b = (double)rgbxel.b / (double)maxpix;
	return rgb;
}

	xel
rgbtoxel(struct rgb rgb, int maxpix) {
/*** NOTE: clips values <0.0 or >1.0 */
	xel rgbxel;
	if (rgb.r > 1.0) {/*fprintf(stderr,"r:%f\n",r);*/ rgb.r = 1.0; }
	if (rgb.g > 1.0) {/*fprintf(stderr,"g:%f\n",g);*/ rgb.g = 1.0; }
	if (rgb.b > 1.0) {/*fprintf(stderr,"b:%f\n",b);*/ rgb.b = 1.0; }
	if (rgb.r < 0.0) {/*fprintf(stderr,"r:%f\n",r);*/ rgb.r = 0.0; }
	if (rgb.g < 0.0) {/*fprintf(stderr,"g:%f\n",g);*/ rgb.g = 0.0; }
	if (rgb.b < 0.0) {/*fprintf(stderr,"b:%f\n",b);*/ rgb.b = 0.0; }
	rgbxel.r = rgb.r * maxpix;
	rgbxel.g = rgb.g * maxpix;
	rgbxel.b = rgb.b * maxpix;
	return rgbxel;
}

	struct rgb **
rgballocarray(int xsize, int ysize, xel **source, int maxpix) {
/*** create two dimensional array. Copy source array if not null, otherwise
 *** fill with black. In case source==null, maxpix is don't care.
 ***/
	int y,x;
	struct rgb **buf = malloc(sizeof(struct rgb *)*ysize);
	if (!buf) { fprintf(stderr,"Cannot malloc!\n"); exit(1); }
	for (y=0; y<ysize; y++) {
		buf[y]=malloc(sizeof(struct rgb)*xsize);
		if (source) {
			for (x=0; x<xsize; x++) {
				buf[y][x] = xeltorgb(source[y][x],maxpix);
			}
		} else {
			for (x=0; x<xsize; x++) {
				buf[y][x].r = 0.0;
				buf[y][x].g = 0.0;
				buf[y][x].b = 0.0;
			}
		}
	}
	return buf;
}

	void
xelarraytorgb(int xsize,int ysize, xel **source, struct rgb **dest,int maxpix){
/*** copy xel array to rgb array
 ***/
	int x,y;
	for (y=0; y<ysize; y++) {
		for (x=0; x<xsize; x++) {
			dest[y][x] = xeltorgb(source[y][x],maxpix);
		}
	}
}

	void
rgbarraytoxel(int xsize,int ysize, struct rgb **source, xel **dest,int maxpix){
/*** copy rgb array to xel array
 ***/
	int x,y;
	for (y=0; y<ysize; y++) {
		for (x=0; x<xsize; x++) {
			dest[y][x] = rgbtoxel(source[y][x],maxpix);
		}
	}
}

/* Originally we tried to comform to CIE Rec. 709.  While some people report
 * gamma values of 2.5, which is a pretty wild function (and others, BTW,
 * reports values as low as 1.6), we should note that CIE Rec. 709
 * approximates a gamma of 1.8 and NOT 2.2  */

/* We tweaked the gammas so that:
 *
 * (1) A colour wheel showing everything from the darkest to the lightest
 * saturated colours contains evenly spaced colours and has monotonous
 * brightness gradient.
 *
 * (2) Hues still look good when saturating pictures. One particularly
 * difficult colour range is found between green and yellow (i.e. oranges and
 * browns), often producing drab yellowish-greenish colours instead of nicer
 * browns that you'd want there.
 *
 * (3) No brightness difference occurs when saturating a not fully saturated
 * colour wheel.
 */

/* with gamma set to 2.2:
 * add 0.1 -> gamma=1.8
 * add 0.3 -> gamma=1.67
 * add 0.4 -> gamma=1.54
 */

#define RGAMMA 1.9
#define GGAMMA 1.9
#define BGAMMA 1.9

#define ADDR 0.2
#define ADDG 0.2
#define ADDB 0.2


#define TBLSIZE 100000
#define DOMAINMAX 3


/* domain: 0..DOMAINMAX */

#define POW_C(chan,v)  pow_tbl  [chan][(int)((v)*(TBLSIZE/DOMAINMAX))]
#define POW_CI(chan,v) pow_i_tbl[chan][(int)((v)*(TBLSIZE/DOMAINMAX))]

static double powmin[3],powmax[3];

/* we only use one channel now, we might have separate tables for r,g,b
 * later on */
static float pow_tbl[1][TBLSIZE];
static float pow_i_tbl[1][TBLSIZE];



	void
stdpnminit() {
/* must be called before using other functions
 */
	int i;
	powmin[0] = pow(ADDR,RGAMMA);
	powmin[1] = pow(ADDG,GGAMMA);
	powmin[2] = pow(ADDB,BGAMMA);
	powmax[0] = pow(1.0+ADDR,RGAMMA);
	powmax[1] = pow(1.0+ADDG,GGAMMA);
	powmax[2] = pow(1.0+ADDB,BGAMMA);
	for (i=0; i<TBLSIZE; i++) {
		pow_tbl  [0][i] =
			pow((double)i/(double)(TBLSIZE/DOMAINMAX),RGAMMA);
		pow_i_tbl[0][i] =
			pow((double)i/(double)(TBLSIZE/DOMAINMAX),1.0/RGAMMA);
		/*pow_tbl  [1][i] =
			pow((double)i/(double)(TBLSIZE/DOMAINMAX),GGAMMA);
		pow_i_tbl[1][i] =
			pow((double)i/(double)(TBLSIZE/DOMAINMAX),1.0/GGAMMA);
		pow_tbl  [2][i] =
			pow((double)i/(double)(TBLSIZE/DOMAINMAX),BGAMMA);
		pow_i_tbl[2][i] =
			pow((double)i/(double)(TBLSIZE/DOMAINMAX),1.0/BGAMMA);*/
	}
}

	double
lineartocierec(double colour,int channel) {
	/*	if (colour <= 0.018) {
			return 4.5 * colour;
		} else {
			return 1.099*pow(colour,0.45) - 0.099;
		}*/
	if (channel==0) {
		colour = POW_CI(0,colour*(powmax[0]-powmin[0])
				+ powmin[0]) - ADDR;
	} else if (channel==1){
		colour = POW_CI(0,colour*(powmax[1]-powmin[1])
				+ powmin[1]) - ADDG;
	} else {
		colour = POW_CI(0,colour*(powmax[2]-powmin[2])
				+ powmin[2]) - ADDB;
	}
	/* take care of table rounding error */
	if (colour < 0.0) colour = 0.0;
	return colour;
}


	double
cierectolinear(double colour,int channel) {
	/*	if (colour <= 0.081) {
			return colour / 4.5;
		} else {
			return pow((colour + 0.099) / 1.099, 1.0/0.45);
		}*/
	if (channel==0) {
		colour = (POW_C(0,(colour+ADDR)) - powmin[0] )
			/ (powmax[0]-powmin[0]);
	} else if (channel==1) {
		colour = (POW_C(0,(colour+ADDG)) - powmin[1] )
			/ (powmax[1]-powmin[1]);
	} else {
		colour = (POW_C(0,(colour+ADDB)) - powmin[2] )
			/ (powmax[2]-powmin[2]);
	}
	/* take care of table rounding error */
	if (colour < 0.0) colour = 0.0;
	return colour;
}


